from collections.abc import Iterable
from math import erf

import numpy as np
import numpy.typing as npt


def hist(
    data: npt.ArrayLike,
    bin_centers: Iterable[float],
) -> npt.NDArray[np.floating]:
    data = np.asarray(data)
    bin_centers = np.asarray(bin_centers)
    diffs = np.diff(bin_centers)
    dx = diffs[0] if np.allclose(diffs, diffs[0]) else None
    if dx is None:
        raise ValueError("`bin_centers` must be equally spaced")
    return np.array(
        [np.mean((x - dx / 2 <= data) & (data < x + dx / 2)) / dx for x in bin_centers],
        dtype=float,
    )


def hist2(
    data1: npt.ArrayLike,
    data2: npt.ArrayLike,
    bin_centers1: Iterable[float],
    bin_centers2: Iterable[float],
) -> npt.NDArray[np.floating]:
    data1 = np.asarray(data1)
    data2 = np.asarray(data2)
    bin_centers1 = np.asarray(bin_centers1)
    bin_centers2 = np.asarray(bin_centers2)
    diffs1 = np.diff(bin_centers1)
    diffs2 = np.diff(bin_centers2)
    dx1 = diffs1[0] if np.allclose(diffs1, diffs1[0]) else None
    dx2 = diffs2[0] if np.allclose(diffs2, diffs2[0]) else None
    if dx1 is None or dx2 is None:
        raise ValueError("`bin_centers1` and `bin_centers2` must be equally spaced")
    return np.array(
        [
            [
                np.mean(
                    (x1 - dx1 / 2 <= data1)
                    & (data1 < x1 + dx1 / 2)
                    & (x2 - dx2 / 2 <= data2)
                    & (data2 < x2 + dx2 / 2)
                )
                / (dx1 * dx2)
                for x2 in bin_centers2
            ]
            for x1 in bin_centers1
        ],
        dtype=float,
    )


@np.vectorize
def phi(x: float) -> float:
    return 0.5 * (1 + erf(x / np.sqrt(2)))


arrowprops = {
    "arrowstyle": "-|>",
    "color": "b",
    "linewidth": 3,
    "shrinkA": 0,
    "shrinkB": 0,
}
